HyperLogLogPlusPlus

Struct HyperLogLogPlusPlus 

pub struct HyperLogLogPlusPlus { /* private fields */ }
HLL++ aggregator for estimating cardinalities of multisets.

The aggregator uses the standard format for storing the internal state of the cardinality estimate as defined in hllplus-unique.proto, allowing users to merge aggregators with data computed in C++ or Go and to load up the cardinalities in a variety of analysis tools.

The precision defines the accuracy of the HLL++ aggregator at the cost of the memory used. The upper bound on the memory required is 2precision bytes, but less memory is used for smaller cardinalities (up to ~2precision - 2). The relative error is 1.04 / sqrt(2precision). A typical value used at Google is 15, which gives an error of about 0.6% while requiring an upper bound of 32 KiB of memory.

Implementations

Source

impl HyperLogLogPlusPlus

pub const MINIMUM_PRECISION: i32 = 10i32

The smallest normal precision supported by this aggregator.

pub const MAXIMUM_PRECISION: i32 = 24i32

The largest normal precision supported by this aggregator.

pub const DEFAULT_NORMAL_PRECISION: i32 = 15i32

The default normal precision that is used if the user does not specify a normal precision.

pub const MAXIMUM_SPARSE_PRECISION: i32 = 25i32

The largest sparse precision supported by this aggregator.

pub const SPARSE_PRECISION_DISABLED: i32 = 0i32

Value used to indicate that the sparse representation should not be used.

pub const DEFAULT_SPARSE_PRECISION_DELTA: i32 = 5i32

If no sparse precision is specified, this value is added to the normal precision to obtain the sparse precision, which optimizes the memory-precision trade-off.

pub const ENCODING_VERSION: i32 = 2i32

The encoding version of the AggregatorStateProto. We only support v2.

pub fn builder() -> HyperLogLogPlusPlusBuilder

Returns a new builder to customize and create a new instance of this aggregator.

pub fn from_proto(proto: AggregatorStateProto) -> Result<Self, SketchError>

Creates a new HyperLogLog++ aggregator from the serialized proto.

The proto must be a valid aggregator state of type AggregatorType::HYPERLOGLOG_PLUS_UNIQUE.

pub fn from_bytes(bytes: &[u8]) -> Result<Self, SketchError>

Creates a new HyperLogLog++ aggregator from the bytes.

The bytes must be a valid serialized AggregatorStateProto of the type AggregatorType::HYPERLOGLOG_PLUS_UNIQUE.

pub fn add_i32(&mut self, value: i32) -> Result<(), SketchError>

Add value to the aggregator.

Returns SketchError if the aggregator is of different type than i32 or u32. See HyperLogLogPlusPlusBuilder::build_for_u32.

pub fn add_u32(&mut self, value: u32) -> Result<(), SketchError>

Add value to the aggregator.

Returns SketchError if the aggregator is of different type than i32 or u32. See HyperLogLogPlusPlusBuilder::build_for_u32.

pub fn add_i64(&mut self, value: i64) -> Result<(), SketchError>

Add value to the aggregator.

Returns SketchError if the aggregator is of different type than i64 or u64. See HyperLogLogPlusPlusBuilder::build_for_u64.

pub fn add_u64(&mut self, value: u64) -> Result<(), SketchError>

Add value to the aggregator.

Returns SketchError if the aggregator is of different type than i64 or u64. See HyperLogLogPlusPlusBuilder::build_for_u64.

pub fn add_bytes(&mut self, value: &[u8]) -> Result<(), SketchError>

Add value to the aggregator.

Returns SketchError if the aggregator is of different type than bytes. See HyperLogLogPlusPlusBuilder::build_for_bytes.

pub fn add_string(&mut self, value: &str) -> Result<(), SketchError>

Add value to the aggregator.

Returns SketchError if the aggregator is of different type than string. See HyperLogLogPlusPlusBuilder::build_for_string.

pub fn normal_precision(&self) -> i32

Returns the normal precision of the aggregator.

pub fn sparse_precision(&self) -> i32

Returns the sparse precision of the aggregator.